@@ -188,9 +188,10 @@ impl Algo {
         match self {
             Algo::Rsa(rsa) => Self::rsa_algo_attrs(rsa),
             Algo::Ecc(ecc) => Self::ecc_algo_attrs(ecc.oid(), ecc.ecc_type()),
-            _ => Err(Error::UnsupportedAlgo(
-                format!("Unexpected Algo {:?}", self).into(),
-            )),
+            _ => Err(Error::UnsupportedAlgo(format!(
+                "Unexpected Algo {:?}",
+                self
+            ))),
         }
     }
 
