@@ -34,7 +34,6 @@ pub struct ApplicationRelatedData(pub(crate) Tlv);
 impl ApplicationRelatedData {
     /// Get application identifier (AID), ISO 7816-4
     pub fn application_id(&self) -> Result<ApplicationIdentifier, Error> {
-        // get from cached "application related data"
         let aid = self.0.find(&[0x4f].into());
 
         if let Some(aid) = aid {
@@ -46,7 +45,6 @@ impl ApplicationRelatedData {
 
     /// Get historical bytes
     pub fn historical_bytes(&self) -> Result<HistoricalBytes, Error> {
-        // get from cached "application related data"
         let hist = self.0.find(&[0x5f, 0x52].into());
 
         if let Some(hist) = hist {
@@ -62,7 +60,6 @@ impl ApplicationRelatedData {
     /// Get extended length information (ISO 7816-4), which
     /// contains maximum number of bytes for command and response.
     pub fn extended_length_information(&self) -> Result<Option<ExtendedLengthInfo>, Error> {
-        // get from cached "application related data"
         let eli = self.0.find(&[0x7f, 0x66].into());
 
         log::trace!("Extended length information: {:x?}", eli);
@@ -88,7 +85,6 @@ impl ApplicationRelatedData {
 
     /// Get extended Capabilities
     pub fn extended_capabilities(&self) -> Result<ExtendedCapabilities, Error> {
-        // FIXME: caching?
         let app_id = self.application_id()?;
         let version = app_id.version();
 
@@ -109,7 +105,6 @@ impl ApplicationRelatedData {
 
     /// Get algorithm attributes (for each key type)
     pub fn algorithm_attributes(&self, key_type: KeyType) -> Result<Algo, Error> {
-        // get from cached "application related data"
         let aa = self.0.find(&[key_type.algorithm_tag()].into());
 
         if let Some(aa) = aa {
@@ -124,7 +119,6 @@ impl ApplicationRelatedData {
 
     /// Get PW status Bytes
     pub fn pw_status_bytes(&self) -> Result<PWStatusBytes, Error> {
-        // get from cached "application related data"
         let psb = self.0.find(&[0xc4].into());
 
         if let Some(psb) = psb {
@@ -143,7 +137,6 @@ impl ApplicationRelatedData {
     /// Fingerprint, per key type.
     /// Zero bytes indicate a not defined private key.
     pub fn fingerprints(&self) -> Result<KeySet<Fingerprint>, Error> {
-        // Get from cached "application related data"
         let fp = self.0.find(&[0xc5].into());
 
         if let Some(fp) = fp {
@@ -157,6 +150,23 @@ impl ApplicationRelatedData {
         }
     }
 
+    pub fn ca_fingerprints(&self) -> Result<[Option<Fingerprint>; 3], Error> {
+        let fp = self.0.find(&[0xc6].into());
+
+        if let Some(fp) = fp {
+            // FIXME: using a KeySet is a weird hack
+            let fp: KeySet<Fingerprint> = (&fp.serialize()[..]).try_into()?;
+
+            let fp = [fp.signature, fp.decryption, fp.authentication];
+
+            log::trace!("CA Fp: {:x?}", fp);
+
+            Ok(fp)
+        } else {
+            Err(Error::NotFound("Failed to get CA fingerprints.".into()))
+        }
+    }
+
     /// Generation dates/times of key pairs
     pub fn key_generation_times(&self) -> Result<KeySet<KeyGenerationTime>, crate::Error> {
         let kg = self.0.find(&[0xcd].into());
@@ -173,6 +183,50 @@ impl ApplicationRelatedData {
             ))
         }
     }
+
+    pub fn key_information(&self) -> Result<Option<KeyInformation>, Error> {
+        let ki = self.0.find(&[0xde].into());
+
+        // TODO: return an error in .into(), if the format of the value is bad
+
+        Ok(ki.map(|v| v.serialize().into()))
+    }
+
+    pub fn uif_pso_cds(&self) -> Result<Option<UIF>, Error> {
+        let uif = self.0.find(&[0xd6].into());
+
+        match uif {
+            None => Ok(None),
+            Some(v) => Ok(Some(v.serialize().try_into()?)),
+        }
+    }
+
+    pub fn uif_pso_dec(&self) -> Result<Option<UIF>, Error> {
+        let uif = self.0.find(&[0xd7].into());
+
+        match uif {
+            None => Ok(None),
+            Some(v) => Ok(Some(v.serialize().try_into()?)),
+        }
+    }
+
+    pub fn uif_pso_aut(&self) -> Result<Option<UIF>, Error> {
+        let uif = self.0.find(&[0xd8].into());
+
+        match uif {
+            None => Ok(None),
+            Some(v) => Ok(Some(v.serialize().try_into()?)),
+        }
+    }
+
+    pub fn uif_attestation(&self) -> Result<Option<UIF>, Error> {
+        let uif = self.0.find(&[0xd9].into());
+
+        match uif {
+            None => Ok(None),
+            Some(v) => Ok(Some(v.serialize().try_into()?)),
+        }
+    }
 }
 
 /// Security support template (see spec pg. 24)
@@ -206,6 +260,260 @@ impl KeyGenerationTime {
     }
 }
 
+/// User Interaction Flag (UIF) (see spec pg. 24)
+#[derive(Clone, Copy, Eq, PartialEq, Debug)]
+pub struct UIF([u8; 2]);
+
+impl TryFrom<Vec<u8>> for UIF {
+    type Error = Error;
+
+    fn try_from(v: Vec<u8>) -> Result<Self, Self::Error> {
+        if v.len() == 2 {
+            Ok(UIF(v.try_into().unwrap()))
+        } else {
+            Err(Error::ParseError(format!("Can't get UID from {:x?}", v)))
+        }
+    }
+}
+
+impl UIF {
+    pub fn touch_policy(&self) -> TouchPolicy {
+        self.0[0].into()
+    }
+
+    pub fn set_touch_policy(&mut self, tm: TouchPolicy) {
+        self.0[0] = tm.into();
+    }
+
+    pub fn features(&self) -> Features {
+        self.0[1].into()
+    }
+
+    pub(crate) fn as_bytes(&self) -> &[u8] {
+        &self.0[..]
+    }
+}
+
+impl Display for UIF {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        write!(
+            f,
+            "Touch policy: {} [Features: {}]",
+            self.touch_policy(),
+            self.features()
+        )
+    }
+}
+
+/// User interaction setting.
+///
+/// See spec pg 24 and https://github.com/Yubico/yubikey-manager/blob/main/ykman/openpgp.py
+#[non_exhaustive]
+pub enum TouchPolicy {
+    Off,
+    On,
+    Fixed,
+    Cached,
+    CachedFixed,
+    Unknown(u8),
+}
+
+impl Display for TouchPolicy {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        match self {
+            TouchPolicy::Off => write!(f, "Off"),
+            TouchPolicy::On => write!(f, "On"),
+            TouchPolicy::Fixed => write!(f, "Fixed"),
+            TouchPolicy::Cached => write!(f, "Cached"),
+            TouchPolicy::CachedFixed => write!(f, "CachedFixed"),
+            TouchPolicy::Unknown(i) => write!(f, "Unknown({})", i),
+        }
+    }
+}
+
+impl From<TouchPolicy> for u8 {
+    fn from(tm: TouchPolicy) -> Self {
+        match tm {
+            TouchPolicy::Off => 0,
+            TouchPolicy::On => 1,
+            TouchPolicy::Fixed => 2,
+            TouchPolicy::Cached => 3,
+            TouchPolicy::CachedFixed => 4,
+            TouchPolicy::Unknown(i) => i,
+        }
+    }
+}
+
+impl From<u8> for TouchPolicy {
+    fn from(i: u8) -> Self {
+        match i {
+            0 => TouchPolicy::Off,
+            1 => TouchPolicy::On,
+            2 => TouchPolicy::Fixed,
+            3 => TouchPolicy::Cached,
+            4 => TouchPolicy::CachedFixed,
+            _ => TouchPolicy::Unknown(i),
+        }
+    }
+}
+
+/// "additional hardware for user interaction" (see spec 4.1.3.2)
+pub struct Features(u8);
+
+impl From<u8> for Features {
+    fn from(i: u8) -> Self {
+        Features(i)
+    }
+}
+
+impl Display for Features {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        let mut ft = vec![];
+
+        if self.0 & 0x80 != 0 {
+            ft.push("Display")
+        }
+        if self.0 & 0x40 != 0 {
+            ft.push("Biometric input sensor")
+        }
+        if self.0 & 0x20 != 0 {
+            ft.push("Button")
+        }
+        if self.0 & 0x10 != 0 {
+            ft.push("Keypad")
+        }
+        if self.0 & 0x8 != 0 {
+            ft.push("LED")
+        }
+        if self.0 & 0x4 != 0 {
+            ft.push("Loudspeaker")
+        }
+        if self.0 & 0x2 != 0 {
+            ft.push("Microphone")
+        }
+        if self.0 & 0x1 != 0 {
+            ft.push("Touchscreen")
+        }
+
+        write!(f, "{}", ft.join(", "))
+    }
+}
+
+pub struct KeyInformation(Vec<u8>);
+
+impl From<Vec<u8>> for KeyInformation {
+    fn from(v: Vec<u8>) -> Self {
+        KeyInformation(v)
+    }
+}
+
+impl KeyInformation {
+    /// How many "additional" keys do we have information for?
+    pub fn num_additional(&self) -> usize {
+        (self.0.len() - 6) / 2
+    }
+
+    fn get_ref(&self, n: usize) -> u8 {
+        self.0[n * 2]
+    }
+    fn get_status(&self, n: usize) -> KeyStatus {
+        self.0[n * 2 + 1].into()
+    }
+
+    pub fn sig_ref(&self) -> u8 {
+        self.get_ref(0)
+    }
+    pub fn sig_status(&self) -> KeyStatus {
+        self.get_status(0)
+    }
+
+    pub fn dec_ref(&self) -> u8 {
+        self.get_ref(1)
+    }
+    pub fn dec_status(&self) -> KeyStatus {
+        self.get_status(1)
+    }
+
+    pub fn aut_ref(&self) -> u8 {
+        self.get_ref(2)
+    }
+    pub fn aut_status(&self) -> KeyStatus {
+        self.get_status(2)
+    }
+
+    pub fn additional_ref(&self, num: usize) -> u8 {
+        self.get_ref(3 + num)
+    }
+    pub fn additional_status(&self, num: usize) -> KeyStatus {
+        self.get_status(3 + num)
+    }
+}
+
+impl Display for KeyInformation {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        writeln!(
+            f,
+            "signature key      (#{}): {}",
+            self.sig_ref(),
+            self.sig_status()
+        )?;
+        writeln!(
+            f,
+            "decryption key     (#{}): {}",
+            self.dec_ref(),
+            self.dec_status()
+        )?;
+        writeln!(
+            f,
+            "authentication key (#{}): {}",
+            self.aut_ref(),
+            self.aut_status()
+        )?;
+
+        for i in 0..self.num_additional() {
+            writeln!(
+                f,
+                "additional key {}   (#{}): {}",
+                i,
+                self.additional_ref(i),
+                self.additional_status(i)
+            )?;
+        }
+
+        Ok(())
+    }
+}
+
+#[non_exhaustive]
+pub enum KeyStatus {
+    NotPresent,
+    Generated,
+    Imported,
+    Unknown(u8),
+}
+
+impl From<u8> for KeyStatus {
+    fn from(i: u8) -> Self {
+        match i {
+            0 => KeyStatus::NotPresent,
+            1 => KeyStatus::Generated,
+            2 => KeyStatus::Imported,
+            _ => KeyStatus::Unknown(i),
+        }
+    }
+}
+
+impl Display for KeyStatus {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        match self {
+            KeyStatus::NotPresent => write!(f, "not present"),
+            KeyStatus::Generated => write!(f, "generated"),
+            KeyStatus::Imported => write!(f, "imported"),
+            KeyStatus::Unknown(i) => write!(f, "unknown status ({})", i),
+        }
+    }
+}
+
 /// 4.2.1 Application Identifier (AID)
 #[derive(Debug, Eq, PartialEq)]
 pub struct ApplicationIdentifier {
