@@ -26,8 +26,18 @@ pub enum Error {
     #[error("Unexpected response length: {0}")]
     ResponseLength(usize),
 
+    #[error("Data not found: {0}")]
+    NotFound(String),
+
+    #[error("Couldn't parse data: {0}")]
+    ParseError(String),
+
+    #[error("Unsupported algorithm: {0}")]
+    UnsupportedAlgo(String),
+
+    // FIXME: placeholder, remove again later?
     #[error("Internal error: {0}")]
-    InternalError(anyhow::Error),
+    InternalError(String),
 }
 
 impl From<StatusBytes> for Error {
@@ -36,12 +46,6 @@ impl From<StatusBytes> for Error {
     }
 }
 
-impl From<anyhow::Error> for Error {
-    fn from(ae: anyhow::Error) -> Self {
-        Error::InternalError(ae)
-    }
-}
-
 /// OpenPGP card "Status Bytes" (ok statuses and errors)
 #[derive(thiserror::Error, Debug, PartialEq, Copy, Clone)]
 #[non_exhaustive]
@@ -129,9 +133,7 @@ impl From<(u8, u8)> for StatusBytes {
             (0x61, bytes) => StatusBytes::OkBytesAvailable(bytes),
 
             (0x62, 0x85) => StatusBytes::TerminationState,
-            (0x63, 0xC0..=0xCF) => {
-                StatusBytes::PasswordNotChecked(status.1 & 0xf)
-            }
+            (0x63, 0xC0..=0xCF) => StatusBytes::PasswordNotChecked(status.1 & 0xf),
             (0x64, 0x02..=0x80) => StatusBytes::TriggeringByCard(status.1),
             (0x65, 0x01) => StatusBytes::MemoryFailure,
             (0x66, 0x00) => StatusBytes::SecurityRelatedIssues,
