@@ -8,7 +8,7 @@ use crate::apdu::commands;
 use crate::apdu::response::RawResponse;
 use crate::card_do::{
     ApplicationRelatedData, CardholderRelatedData, Fingerprint, KeyGenerationTime, Lang,
-    PWStatusBytes, SecuritySupportTemplate, Sex,
+    PWStatusBytes, SecuritySupportTemplate, Sex, UIF,
 };
 use crate::crypto_data::{CardUploadableKey, Cryptogram, Hash, PublicKeyMaterial};
 use crate::tlv::{value::Value, Tlv};
@@ -85,36 +85,6 @@ impl<'a> OpenPgpTransaction<'a> {
         self.tx.application_related_data()
     }
 
-    // #[allow(dead_code)]
-    // fn ca_fingerprints() {
-    //     unimplemented!()
-    // }
-    //
-    // #[allow(dead_code)]
-    // fn key_information() {
-    //     unimplemented!()
-    // }
-    //
-    // #[allow(dead_code)]
-    // fn uif_pso_cds() {
-    //     unimplemented!()
-    // }
-    //
-    // #[allow(dead_code)]
-    // fn uif_pso_dec() {
-    //     unimplemented!()
-    // }
-    //
-    // #[allow(dead_code)]
-    // fn uif_pso_aut() {
-    //     unimplemented!()
-    // }
-    //
-    // #[allow(dead_code)]
-    // fn uif_attestation() {
-    //     unimplemented!()
-    // }
-
     // --- login data (5e) ---
 
     /// Get URL (5f50)
@@ -248,24 +218,6 @@ impl<'a> OpenPgpTransaction<'a> {
         Ok(resp.data()?.to_vec())
     }
 
-    /// Set data of "private use" DO.
-    ///
-    /// `num` must be between 1 and 4.
-    ///
-    /// Access condition:
-    /// - 1/3 need PW1 (82)
-    /// - 2/4 need PW3
-    pub fn set_private_use_do(&mut self, num: u8, data: Vec<u8>) -> Result<Vec<u8>, Error> {
-        log::info!("OpenPgpTransaction: set_private_use_do");
-
-        assert!((1..=4).contains(&num));
-
-        let cmd = commands::put_private_use_do(num, data);
-        let resp = apdu::send_command(self.tx(), cmd, true)?;
-
-        Ok(resp.data()?.to_vec())
-    }
-
     // ----------
 
     /// Reset all state on this OpenPGP card.
@@ -626,7 +578,25 @@ impl<'a> OpenPgpTransaction<'a> {
         Ok(resp.data().map(|d| d.to_vec())?)
     }
 
-    // --- admin ---
+    // --- PUT DO ---
+
+    /// Set data of "private use" DO.
+    ///
+    /// `num` must be between 1 and 4.
+    ///
+    /// Access condition:
+    /// - 1/3 need PW1 (82)
+    /// - 2/4 need PW3
+    pub fn set_private_use_do(&mut self, num: u8, data: Vec<u8>) -> Result<Vec<u8>, Error> {
+        log::info!("OpenPgpTransaction: set_private_use_do");
+
+        assert!((1..=4).contains(&num));
+
+        let cmd = commands::put_private_use_do(num, data);
+        let resp = apdu::send_command(self.tx(), cmd, true)?;
+
+        Ok(resp.data()?.to_vec())
+    }
 
     pub fn set_name(&mut self, name: &[u8]) -> Result<(), Error> {
         log::info!("OpenPgpTransaction: set_name");
@@ -661,59 +631,6 @@ impl<'a> OpenPgpTransaction<'a> {
         apdu::send_command(self.tx(), put_url, false)?.try_into()
     }
 
-    pub fn set_creation_time(
-        &mut self,
-        time: KeyGenerationTime,
-        key_type: KeyType,
-    ) -> Result<(), Error> {
-        log::info!("OpenPgpTransaction: set_creation_time");
-
-        // Timestamp update
-        let time_value: Vec<u8> = time
-            .get()
-            .to_be_bytes()
-            .iter()
-            .skip_while(|&&e| e == 0)
-            .copied()
-            .collect();
-
-        let time_cmd = commands::put_data(&[key_type.timestamp_put_tag()], time_value);
-
-        apdu::send_command(self.tx(), time_cmd, false)?.try_into()
-    }
-
-    pub fn set_fingerprint(&mut self, fp: Fingerprint, key_type: KeyType) -> Result<(), Error> {
-        log::info!("OpenPgpTransaction: set_fingerprint");
-
-        let fp_cmd = commands::put_data(&[key_type.fingerprint_put_tag()], fp.as_bytes().to_vec());
-
-        apdu::send_command(self.tx(), fp_cmd, false)?.try_into()
-    }
-
-    /// Set PW Status Bytes.
-    ///
-    /// If `long` is false, send 1 byte to the card, otherwise 4.
-    /// According to the spec, length information should not be changed.
-    ///
-    /// So, effectively, with 'long == false' the setting `pw1_cds_multi`
-    /// can be changed.
-    /// With 'long == true', the settings `pw1_pin_block` and `pw3_pin_block`
-    /// can also be changed.
-    ///
-    /// (See OpenPGP card spec, pg. 28)
-    pub fn set_pw_status_bytes(
-        &mut self,
-        pw_status: &PWStatusBytes,
-        long: bool,
-    ) -> Result<(), Error> {
-        log::info!("OpenPgpTransaction: set_pw_status_bytes");
-
-        let data = pw_status.serialize_for_put(long);
-
-        let cmd = commands::put_pw_status(data);
-        apdu::send_command(self.tx(), cmd, false)?.try_into()
-    }
-
     /// Set cardholder certificate (for AUT, DEC or SIG).
     ///
     /// Call select_data() before calling this fn, to select a particular
@@ -740,6 +657,84 @@ impl<'a> OpenPgpTransaction<'a> {
         apdu::send_command(self.tx(), cmd, false)?.try_into()
     }
 
+    /// Set PW Status Bytes.
+    ///
+    /// If `long` is false, send 1 byte to the card, otherwise 4.
+    /// According to the spec, length information should not be changed.
+    ///
+    /// So, effectively, with 'long == false' the setting `pw1_cds_multi`
+    /// can be changed.
+    /// With 'long == true', the settings `pw1_pin_block` and `pw3_pin_block`
+    /// can also be changed.
+    ///
+    /// (See OpenPGP card spec, pg. 28)
+    pub fn set_pw_status_bytes(
+        &mut self,
+        pw_status: &PWStatusBytes,
+        long: bool,
+    ) -> Result<(), Error> {
+        log::info!("OpenPgpTransaction: set_pw_status_bytes");
+
+        let data = pw_status.serialize_for_put(long);
+
+        let cmd = commands::put_pw_status(data);
+        apdu::send_command(self.tx(), cmd, false)?.try_into()
+    }
+
+    pub fn set_fingerprint(&mut self, fp: Fingerprint, key_type: KeyType) -> Result<(), Error> {
+        log::info!("OpenPgpTransaction: set_fingerprint");
+
+        let fp_cmd = commands::put_data(&[key_type.fingerprint_put_tag()], fp.as_bytes().to_vec());
+
+        apdu::send_command(self.tx(), fp_cmd, false)?.try_into()
+    }
+
+    pub fn set_ca_fingerprint_1(&mut self, fp: Fingerprint) -> Result<(), Error> {
+        log::info!("OpenPgpTransaction: set_ca_fingerprint_1");
+
+        let fp_cmd = commands::put_data(&[0xCA], fp.as_bytes().to_vec());
+        apdu::send_command(self.tx(), fp_cmd, false)?.try_into()
+    }
+
+    pub fn set_ca_fingerprint_2(&mut self, fp: Fingerprint) -> Result<(), Error> {
+        log::info!("OpenPgpTransaction: set_ca_fingerprint_2");
+
+        let fp_cmd = commands::put_data(&[0xCB], fp.as_bytes().to_vec());
+        apdu::send_command(self.tx(), fp_cmd, false)?.try_into()
+    }
+
+    pub fn set_ca_fingerprint_3(&mut self, fp: Fingerprint) -> Result<(), Error> {
+        log::info!("OpenPgpTransaction: set_ca_fingerprint_3");
+
+        let fp_cmd = commands::put_data(&[0xCC], fp.as_bytes().to_vec());
+        apdu::send_command(self.tx(), fp_cmd, false)?.try_into()
+    }
+
+    pub fn set_creation_time(
+        &mut self,
+        time: KeyGenerationTime,
+        key_type: KeyType,
+    ) -> Result<(), Error> {
+        log::info!("OpenPgpTransaction: set_creation_time");
+
+        // Timestamp update
+        let time_value: Vec<u8> = time
+            .get()
+            .to_be_bytes()
+            .iter()
+            .skip_while(|&&e| e == 0)
+            .copied()
+            .collect();
+
+        let time_cmd = commands::put_data(&[key_type.timestamp_put_tag()], time_value);
+
+        apdu::send_command(self.tx(), time_cmd, false)?.try_into()
+    }
+
+    // FIXME: optional DO SM-Key-ENC
+
+    // FIXME: optional DO SM-Key-MAC
+
     /// Set resetting code
     /// (4.3.4 Resetting Code)
     pub fn set_resetting_code(&mut self, resetting_code: &[u8]) -> Result<(), Error> {
@@ -749,6 +744,46 @@ impl<'a> OpenPgpTransaction<'a> {
         apdu::send_command(self.tx(), cmd, false)?.try_into()
     }
 
+    // FIXME: optional DO for PSO:ENC/DEC with AES
+
+    /// Set UIF for PSO:CDS
+    pub fn set_uif_pso_cds(&mut self, uif: &UIF) -> Result<(), Error> {
+        log::info!("OpenPgpTransaction: set_uif_pso_cds");
+
+        let cmd = commands::put_data(&[0xd6], uif.as_bytes().to_vec());
+        apdu::send_command(self.tx(), cmd, false)?.try_into()
+    }
+
+    /// Set UIF for PSO:DEC
+    pub fn set_uif_pso_dec(&mut self, uif: &UIF) -> Result<(), Error> {
+        log::info!("OpenPgpTransaction: set_uif_pso_dec");
+
+        let cmd = commands::put_data(&[0xd7], uif.as_bytes().to_vec());
+        apdu::send_command(self.tx(), cmd, false)?.try_into()
+    }
+
+    /// Set UIF for PSO:AUT
+    pub fn set_uif_pso_aut(&mut self, uif: &UIF) -> Result<(), Error> {
+        log::info!("OpenPgpTransaction: set_uif_pso_aut");
+
+        let cmd = commands::put_data(&[0xd8], uif.as_bytes().to_vec());
+        apdu::send_command(self.tx(), cmd, false)?.try_into()
+    }
+
+    // FIXME: UIF for Attestation key and Generate Attestation command (Yubico)
+
+    // FIXME: Attestation key algo attr, FP, CA-FP, creation time
+
+    // FIXME: SM keys (ENC and MAC) with Tags D1 and D2
+
+    // FIXME: KDF DO
+
+    // FIXME: certificate used with secure messaging
+
+    // FIXME: Attestation Certificate (Yubico)
+
+    // -----------------
+
     /// Import an existing private key to the card.
     /// (This implicitly sets the algorithm info, fingerprint and timestamp)
     pub fn key_import(
