@@ -3,8 +3,6 @@
 
 //! Value in a TLV data structure
 
-use anyhow::Result;
-
 use crate::card_do::complete;
 use crate::tlv::Tlv;
 
@@ -19,10 +17,7 @@ pub enum Value {
 }
 
 impl Value {
-    pub(crate) fn parse(
-        data: &[u8],
-        constructed: bool,
-    ) -> nom::IResult<&[u8], Self> {
+    pub(crate) fn parse(data: &[u8], constructed: bool) -> nom::IResult<&[u8], Self> {
         match constructed {
             false => Ok((&[], Value::S(data.to_vec()))),
             true => {
@@ -40,7 +35,7 @@ impl Value {
         }
     }
 
-    pub fn from(data: &[u8], constructed: bool) -> Result<Self> {
+    pub fn from(data: &[u8], constructed: bool) -> Result<Self, crate::Error> {
         complete(Self::parse(data, constructed))
     }
 
